<?php
/**
 * @package    Joomla.Component.Builder
 *
 * @created    4th September, 2022
 * @author     Llewellyn van der Merwe <https://dev.vdm.io>
 * @git        Joomla Component Builder <https://git.vdm.dev/joomla/Component-Builder>
 * @copyright  Copyright (C) 2015 Vast Development Method. All rights reserved.
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 */

namespace VDM\Joomla\Componentbuilder\Package\CustomCode\Readme;


use VDM\Joomla\Interfaces\Readme\ItemInterface;


/**
 * Compiler Custom Code Item Readme
 * 
 * @since  5.1.1
 */
final class Item implements ItemInterface
{
	/**
	 * Generate a README for a JCB Custom Code block in Markdown format.
	 *
	 * Includes system name, execution target (auto/manual), path or function name, and code body.
	 *
	 * @param  object  $item  The custom code definition.
	 *
	 * @return string  The generated README.
	 * @since  5.1.1
	 */
	public function get(object $item): string
	{
		$readme = [];

		// Title
		$readme[] = '### JCB! Custom Code';
		$readme[] = '# ' . ($item->system_name ?? 'error: missing system name');
		$readme[] = '';

		$target = (int) ($item->target ?? 0);

		if ($target === 1)
		{
			// Hash-based automation
			$readme[] = '## Hash (automation)';

			$commentType     = (int) ($item->comment_type ?? 0);
			$joomlaVersion   = (int) ($item->joomla_version ?? 0);
			$joomlaLabel     = $joomlaVersion > 2 ? " J{$joomlaVersion}" : '';
			$insertionType   = $commentType === 1 ? "Replacement" : "Insertion";
			$readme[] = "#### {$insertionType}{$joomlaLabel}";
			$readme[] = '';

			$path = $item->path ?? 'error: missing path';
			$readme[] = '### Path';
			$readme[] = '```';
			$readme[] = $path;
			$readme[] = '```';
			$readme[] = '';
		}
		elseif ($target === 2 && !empty($item->function_name))
		{
			// Manual [CUSTOMCODE=...] style usage
			$readme[] = '## JCB (manual)';
			$readme[] = '```';
			$readme[] = '[CUSTOM' . 'CODE=' . $item->function_name . ']';
			$readme[] = '```';
			$readme[] = '';
		}

		// Code block
		$code = $item->code ?? 'error: missing code';
		$readme[] = '### Code';
		$readme[] = '```php';
		$readme[] = $code;
		$readme[] = '```';
		$readme[] = '';

		// Footer
		$readme[] = '> Add clean, self-contained code into your components with this reusable custom-code snippet designed for seamless integration and easy updates in JCB.';
		$readme[] = '';

		$readme[] = <<<MD
### Used in [Joomla Component Builder](https://www.joomlacomponentbuilder.com) - [Source](https://git.vdm.dev/joomla/Component-Builder) - [Mirror](https://github.com/vdm-io/Joomla-Component-Builder) - [Download](https://git.vdm.dev/joomla/pkg-component-builder/releases)

---
[![Joomla Volunteer Portal](https://img.shields.io/badge/-Joomla-gold?logo=joomla)](https://volunteers.joomla.org/joomlers/1396-llewellyn-van-der-merwe "Join Llewellyn on the Joomla Volunteer Portal: Shaping the Future Together!") [![Octoleo](https://img.shields.io/badge/-Octoleo-black?logo=linux)](https://git.vdm.dev/octoleo "--quiet") [![Llewellyn](https://img.shields.io/badge/-Llewellyn-ffffff?logo=gitea)](https://git.vdm.dev/Llewellyn "Collaborate and Innovate with Llewellyn on Git: Building a Better Code Future!") [![Telegram](https://img.shields.io/badge/-Telegram-blue?logo=telegram)](https://t.me/Joomla_component_builder "Join Llewellyn and the Community on Telegram: Building Joomla Components Together!") [![Mastodon](https://img.shields.io/badge/-Mastodon-9e9eec?logo=mastodon)](https://joomla.social/@llewellyn "Connect and Engage with Llewellyn on Joomla Social: Empowering Communities, One Post at a Time!") [![X (Twitter)](https://img.shields.io/badge/-X-black?logo=x)](https://x.com/llewellynvdm "Join the Conversation with Llewellyn on X: Where Ideas Take Flight!") [![GitHub](https://img.shields.io/badge/-GitHub-181717?logo=github)](https://github.com/Llewellynvdm "Build, Innovate, and Thrive with Llewellyn on GitHub: Turning Ideas into Impact!") [![YouTube](https://img.shields.io/badge/-YouTube-ff0000?logo=youtube)](https://www.youtube.com/@OctoYou "Explore, Learn, and Create with Llewellyn on YouTube: Your Gateway to Inspiration!") [![n8n](https://img.shields.io/badge/-n8n-black?logo=n8n)](https://n8n.io/creators/octoleo "Effortless Automation and Impactful Workflows with Llewellyn on n8n!") [![Docker Hub](https://img.shields.io/badge/-Docker-grey?logo=docker)](https://hub.docker.com/u/llewellyn "Llewellyn on Docker: Containerize Your Creativity!") [![Open Collective](https://img.shields.io/badge/-Donate-green?logo=opencollective)](https://opencollective.com/joomla-component-builder "Donate towards JCB: Help Llewellyn financially so he can continue developing this great tool!") [![GPG Key](https://img.shields.io/badge/-GPG-blue?logo=gnupg)](https://git.vdm.dev/Llewellyn/gpg "Unlock Trust and Security with Llewellyn's GPG Key: Your Gateway to Verified Connections!")
MD;

		return implode("\n", $readme);
	}

}

